var _DISPLAYS;

// map the DSL standard to JS: https://docs.mpi-internal.com/scmspain/all--lib-form-builder-docs/form-specification/field/
import { pickFieldById, fieldsNamesInOrderOfDefinition } from '../reducer/fields';
var FIELDS = {
  TEXT: 'text',
  NUMERIC: 'numeric',
  FIELDSET: 'fieldset',
  PICKER: 'picker'
};
var DISPLAYS = (_DISPLAYS = {}, _DISPLAYS[FIELDS.TEXT] = {
  MULTILINE: 'multiline',
  EMAIL: 'email',
  PHONE: 'phone',
  // not part of the DSL standard
  TEXT: 'text',
  // not part of the DSL standard
  DEFAULT: ''
}, _DISPLAYS[FIELDS.NUMERIC] = {
  MONEY: 'money',
  SURFACE: 'surface',
  DEFAULT: ''
}, _DISPLAYS[FIELDS.PICKER] = {
  SWITCH: 'switch',
  AUTOCOMPLETE: 'autocomplete',
  DROPDOWN: 'dropdown',
  RADIO: 'radio',
  CHECKBOX: 'checkbox',
  BUTTON_INLINE: 'button-inline'
}, _DISPLAYS);
var CONSTRAINTS = {
  notnull: 'valueMissing',
  pattern: 'patternMismatch',
  maxlength: 'tooLong',
  minlength: 'tooShort'
};

var checkConstraintsFromField = function checkConstraintsFromField(field) {
  var errorMessages = [];
  var constraints = field.constraints || [];
  var elementNode = document.getElementById(field.id);
  var elementValidity = elementNode == null ? void 0 : elementNode.validity; // if element has no validity is not a form element
  // if element has not constraints there is no need to validate

  if (elementValidity && !!constraints) {
    errorMessages = constraints.filter(function (constraint) {
      // for each constraint property defined in the field
      var validityRule = CONSTRAINTS[Object.keys(constraint == null ? void 0 : constraint.property)[0]]; // if true, the constraint is not satisfied, so add the constraint to errors

      if (elementValidity[validityRule]) return true;
      return false;
    }) // return
    .map(function (constraint) {
      return constraint.message;
    });
  } // custom validation: minlength constraint in html textarea is not working as expected, need to handle it manually


  if ( // if is a field text and display multiline
  field.type === FIELDS.TEXT && field.display === DISPLAYS[FIELDS.TEXT].MULTILINE && // if element validity is tooShort means that it is working as expected and it is not required manual validation
  !elementValidity.tooShort) {
    var _textAreaHasMinLength;

    var textAreaHasMinLength = field.constraints.find(function (constraint) {
      var _constraint$property;

      return (_constraint$property = constraint.property) == null ? void 0 : _constraint$property.minlength;
    });
    var textAreaValue = elementNode == null ? void 0 : elementNode.value;
    if ((textAreaValue == null ? void 0 : textAreaValue.length) < parseInt(textAreaHasMinLength == null ? void 0 : (_textAreaHasMinLength = textAreaHasMinLength.property) == null ? void 0 : _textAreaHasMinLength.minlength)) errorMessages = [].concat(errorMessages, [textAreaHasMinLength.message]);
  } // custom validation: pattern constraint in html input (type=checkbox) is not natively supported, need to handle it manually


  if (field.type === FIELDS.PICKER && field.display === DISPLAYS[FIELDS.PICKER].CHECKBOX) {
    var checkboxValue = JSON.parse(elementNode == null ? void 0 : elementNode.value);
    var checkboxShouldBeTrueConstraint = field.constraints.find(function (constraint) {
      var _constraint$property2;

      return ((_constraint$property2 = constraint.property) == null ? void 0 : _constraint$property2.pattern) === '^true$';
    }); // if should be true according to field, and is false show error message

    if (checkboxShouldBeTrueConstraint && !checkboxValue) errorMessages = [checkboxShouldBeTrueConstraint.message].concat(errorMessages);
  }

  return errorMessages;
};

var checkConstrainstsFactory = function checkConstrainstsFactory(json) {
  return function (_ref) {
    var fieldID = _ref["for"],
        all = _ref.all;
    var fieldsToValidate = [];

    if (all && fieldID) {
      window.console.warn('[form/builder]: checkConstrainstsFactory: both modes validate all fields and validate a concrete field are not compatible, please use one of them');
    } else if (all) {
      var _json$form;

      debugger; // eslint-disable-line

      fieldsToValidate = fieldsNamesInOrderOfDefinition(json == null ? void 0 : (_json$form = json.form) == null ? void 0 : _json$form.fields);
    } else if (fieldID) {
      fieldsToValidate = [fieldID];
    } else {
      window.console.warn('[form/builder]: checkConstrainstsFactory: Specify if you want to validate a specific field or all the fields');
    }

    var fieldsWithErrors = {};
    fieldsToValidate.forEach(function (fieldId) {
      var field = pickFieldById(json.form.fields, fieldId);
      fieldsWithErrors[field.id] = checkConstraintsFromField(field);
    });
    return fieldsWithErrors;
  };
};

export { FIELDS, DISPLAYS, CONSTRAINTS, checkConstrainstsFactory };